@@ -1078,17 +1078,21 @@ dojo.declare("dojox.form.FileUploader", [dijit._Widget, dijit._Templated, dijit.
 		// summary:
 		//		Build the form that holds the fileInput
 		//
+
 		if(this._formNode){ return; }
 
-		if(dojo.isIE){
+		if(dojo.isIE < 9 || (dojo.isIE && dojo.isQuirks)){
 			this._formNode = document.createElement('<form enctype="multipart/form-data" method="post">');
 			this._formNode.encoding = "multipart/form-data";
+			this._formNode.id = dijit.getUniqueId("FileUploaderForm"); // needed for dynamic style
+			this.domNode.appendChild(this._formNode);
 		}else{
-			this._formNode = document.createElement('form');
-			this._formNode.setAttribute("enctype", "multipart/form-data");
+			this._formNode = dojo.create('form', {
+				enctype:"multipart/form-data",
+				method:"post",
+				id:dijit.getUniqueId("FileUploaderForm")
+			}, this.domNode);
 		}
-		this._formNode.id = dijit.getUniqueId("FileUploaderForm"); // needed for dynamic style
-		this.domNode.appendChild(this._formNode);
 	},
 
 	_buildFileInput: function(){
