@@ -197,8 +197,7 @@ doh.extend(doh.Deferred, {
 			if(this.fired == -1){
 				this.errback(new Error("Deferred(unfired)"));
 			}
-		}else if(this.fired == 0 &&
-					(this.results[0] instanceof doh.Deferred)){
+		}else if(this.fired == 0 && this.results[0] && this.results[0].cancel){
 			this.results[0].cancel();
 		}
 	},
@@ -297,7 +296,7 @@ doh.extend(doh.Deferred, {
 			try {
 				res = f(res);
 				fired = ((res instanceof Error) ? 1 : 0);
-				if(res instanceof doh.Deferred){
+				if(res && res.addCallback){
 					cb = function(res){
 						self._continue(res);
 					};
@@ -915,9 +914,9 @@ doh._runPerfFixture = function(/*String*/groupName, /*Object*/fixture){
 							state.countdown--;
 							if(state.countdown){
 								var ret = fixture.runTest(doh);
-								if(ret instanceof doh.Deferred){
-									//Deferreds have to be handled async,
-									//otherwise we just keep looping.
+								if(ret && ret.addCallback){
+									// Deferreds have to be handled async,
+									// otherwise we just keep looping.
 									var atState = {
 										countdown: state.countdown
 									};
@@ -1015,7 +1014,7 @@ doh._calcTrialIterations =  function(/*String*/ groupName, /*Object*/ fixture){
 				if(state.curIter < state.iterations){
 					try{
 						var ret = testFunc(doh);
-						if(ret instanceof doh.Deferred){
+						if(ret && ret.addCallback){
 							var aState = {
 								start: state.start,
 								curIter: state.curIter + 1,
@@ -1080,7 +1079,7 @@ doh._runRegFixture = function(/*String*/groupName, /*Object*/fixture){
 	// if we get a deferred back from the test runner, we know we're
 	// gonna wait for an async result. It's up to the test code to trap
 	// errors and give us an errback or callback.
-	if(ret instanceof doh.Deferred){
+	if(ret && ret.addCallback){
 		tg.inFlight++;
 		ret.groupName = groupName;
 		ret.fixture = fixture;
@@ -1446,7 +1445,7 @@ tests = doh;
 		}
 	}catch(e){
 		print("\n"+doh._line);
-		print("The Dojo Unit Test Harness, $Rev: 22059 $");
+		print("The Dojo Unit Test Harness, $Rev: 26956 $");
 		print("Copyright (c) 2010, The Dojo Foundation, All Rights Reserved");
 		print(doh._line, "\n");
 
