@@ -15,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Markup
  * @subpackage Renderer
- * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: RendererAbstract.php 20662 2010-01-26 18:40:33Z kokx $
+ * @version    $Id: RendererAbstract.php 24594 2012-01-05 21:27:01Z matthew $
  */
 
 /**
@@ -39,7 +39,7 @@ require_once 'Zend/Markup/Renderer/TokenConverterInterface.php';
  * @category   Zend
  * @package    Zend_Markup
  * @subpackage Renderer
- * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 abstract class Zend_Markup_Renderer_RendererAbstract
@@ -131,9 +131,6 @@ abstract class Zend_Markup_Renderer_RendererAbstract
         if (isset($options['parser'])) {
             $this->setParser($options['parser']);
         }
-        if (isset($options['useDefaultTags']) && ($options['useDefaultTags'] === false)) {
-            $this->removeDefaultTags();
-        }
         if (!isset($options['useDefaultFilters']) || ($options['useDefaultFilters'] === true)) {
             $this->addDefaultFilters();
         }
@@ -179,13 +176,11 @@ abstract class Zend_Markup_Renderer_RendererAbstract
      *
      * @param string $encoding
      *
-     * @return Zend_Markup_Renderer_RendererAbstract
+     * @return void
      */
     public static function setEncoding($encoding)
     {
         self::$_encoding = $encoding;
-
-        return $this;
     }
 
     /**
@@ -557,7 +552,7 @@ abstract class Zend_Markup_Renderer_RendererAbstract
     public function getDefaultFilter()
     {
         if (null === $this->_defaultFilter) {
-            $this->setDefaultFilter();
+            $this->addDefaultFilters();
         }
 
         return $this->_defaultFilter;
@@ -669,6 +664,34 @@ abstract class Zend_Markup_Renderer_RendererAbstract
         return $this;
     }
 
+    /**
+     * Add a render group
+     *
+     * @param string $name
+     * @param array $allowedInside
+     * @param array $allowsInside
+     *
+     * @return void
+     */
+    public function addGroup($name, array $allowedInside = array(), array $allowsInside = array())
+    {
+        $this->_groups[$name] = $allowsInside;
+
+        foreach ($allowedInside as $group) {
+            $this->_groups[$group][] = $name;
+        }
+    }
+
+    /**
+     * Get group definitions
+     *
+     * @return array
+     */
+    public function getGroups()
+    {
+        return $this->_groups;
+    }
+
     /**
      * Set the default filters
      *
