@@ -15,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Navigation
  * @subpackage UnitTests
- * @copyright  Copyright (c) 2005-2011 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: ContainerTest.php 23775 2011-03-01 17:25:24Z ralph $
+ * @version    $Id: ContainerTest.php 24594 2012-01-05 21:27:01Z matthew $
  */
 
 require_once 'Zend/Navigation.php';
@@ -29,7 +29,7 @@ require_once 'Zend/Config.php';
  * @category   Zend
  * @package    Zend_Navigation
  * @subpackage UnitTests
- * @copyright  Copyright (c) 2005-2011 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  * @group      Zend_Navigation
  */
@@ -367,6 +367,20 @@ class Zend_Navigation_ContainerTest extends PHPUnit_Framework_TestCase
                             'Expected 3 pages, found ' . count($nav));
     }
 
+    /**
+     * @group ZF-9815
+     */
+    public function testAddPagesShouldWorkWithNavigationContainer()
+    {        
+        $nav = new Zend_Navigation();
+        $nav->addPages($this->_getFindByNavigation());
+        
+        $this->assertEquals(3, count($nav),
+                            'Expected 3 pages, found ' . count($nav));
+        
+        $this->assertEquals($nav->toArray(), $this->_getFindByNavigation()->toArray());
+    }
+
     public function testAddPagesShouldThrowExceptionWhenGivenString()
     {
         $nav = new Zend_Navigation();
